use telers::{
    event::{telegram::HandlerResult, EventReturn},
    Bot,
};

use tokio::time::{sleep, Duration as DurationSleep};

use crate::{
    types::{
        enums::target_user::TargetUser,
        structs::handler_entity::{ExtractedEntityData, HandlerEntity},
    },
    utils::telegram::{ban_member::ban_chat_member, try_do::try_restrict},
};

pub async fn ban_member(
    handler_entity: HandlerEntity,
    chat_id: i64,
    user: TargetUser,
    delay: u64,
) -> HandlerResult {
    let (bot, message, mut sender_builder): ExtractedEntityData = handler_entity.extract();

    let user_id: i64 = match user.get_id() {
        Some(id) => id,
        None => {
            sender_builder
                .text("Ответьте на сообщение участника чата, которого вы хотите забанить")
                .reply_to(message.id())
                .build()
                .send(&bot)
                .await
                .unwrap();
            return Ok(EventReturn::Cancel);
        }
    };

    sleep(DurationSleep::from_millis(delay)).await;

    let callback = || async { ban_chat_member(&bot, user_id, chat_id).await };
    let demote_args: (&Bot, i64, i64) = (&bot, user_id, chat_id);
    sender_builder
        .set_text("Невозможно забанить участника чата, демотните своими силами и попробуйте снова");

    if try_restrict(callback, demote_args, sender_builder.clone().build())
        .await
        .is_err()
    {
        sender_builder.build().send(&bot).await?;
        Ok(EventReturn::Cancel)
    } else {
        let banned_user_name: String = user.get_user_name(&bot, &message).await.unwrap();

        sender_builder
            .reply_to(message.id())
            .text(format!("Пользователь {} забанен.", banned_user_name))
            .build()
            .send(&bot)
            .await?;

        Ok(EventReturn::Finish)
    }
}
